/**
 * 
 */
package net.bigeon.gclc.swt;

import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

import net.bigeon.gclc.utils.AOutputForwardRunnable;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** The manager for console output to insert in a text.
 * 
 * @author Emmanuel Bigeon */
public final class ConsoleOutputManager implements ConsoleOutputDisplay {
    /** The local implementation of the forwarding runnable.
     *
     * @author Emmanuel Bigeon */
    public static final class ToSWTConsoleForwardRunnable extends AOutputForwardRunnable {
        /** The running status */
        private boolean                    running = true;
        private final PipedConsoleOutput   out;
        private final ConsoleOutputDisplay display;
        private final Widget               element;

        /** @param manager the manager
         * @param display the display
         * @param element the composite */
        public ToSWTConsoleForwardRunnable(final PipedConsoleOutput manager,
                                           final ConsoleOutputDisplay display,
                                           final Widget element) {
            super(manager);
            out = manager;
            this.display = display;
            this.element = element;
        }

        @Override
        protected void forwardLine(final String m) {
            display.appendLine(m);
        }

        @Override
        protected boolean isRunning() {
            return running && !element.isDisposed();
        }

        /** @param running the running to set */
        public void setRunning(final boolean running) {
            this.running = running;
        }

        /** @return the currently forwarded output */
        public PipedConsoleOutput getOuput() {
            return out;
        }
    }

    private final Text                  text;
    /** The forwarding runnable */
    private ToSWTConsoleForwardRunnable forward;
    private Thread                      forwardThread;

    /** @param text the text to display the output in. */
    public ConsoleOutputManager(final Text text) {
        super();
        this.text = text;
    }

    /** @param next the next message */
    @Override
    public void appendLine(final String next) {
        text.getDisplay().syncExec(new Runnable() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                if (!text.getText().isEmpty()) {
                    text.append(System.lineSeparator());
                }
                text.append(next);
            }
        });
    }

    /** Set the output.
     *
     * @param output the output to set */
    public void setManager(final PipedConsoleOutput output) {
        if (forward != null) {
            if (forward.getOuput() == output) {
                return;
            }
            forward.setRunning(false);
        }
        if (output == null) {
            forward = null;
            forwardThread = null;
            return;
        }
        forward = new ToSWTConsoleForwardRunnable(output, this, text);
        forwardThread = new Thread(forward, "gclcToSWT"); //$NON-NLS-1$
        forwardThread.start();
    }

    /** @return the forwardThread */
    public Thread getForwardThread() {
        return forwardThread;
    }
}
